@@ -26,4 +26,10 @@ fi
 if [[ ! -d score-avnd-granola ]]; then
   git clone --recursive -j16 https://github.com/bltzr/score-avnd-granola
 fi
+
+if [[ ! -d score-addon-ultraleap ]]; then
+  git clone --recursive -j16 https://github.com/ossia/score-addon-ultraleap
+fi
 )
+
+
