@@ -18,6 +18,7 @@
 #include <avnd/common/for_nth.hpp>
 #include <avnd/concepts/file_port.hpp>
 #include <avnd/concepts/gfx.hpp>
+#include <avnd/concepts/temporality.hpp>
 #include <avnd/concepts/ui.hpp>
 #include <avnd/introspection/messages.hpp>
 #include <avnd/wrappers/bus_host_process_adapter.hpp>
@@ -165,9 +166,14 @@ struct Metadata<Process::ProcessFlags_k, oscr::ProcessModel<Info>>
     }
     else
     {
-      return Process::ProcessFlags(
+      Process::ProcessFlags flags = Process::ProcessFlags(
           Process::ProcessFlags::SupportsLasting
           | Process::ProcessFlags::ControlSurface);
+
+      if constexpr(avnd::tag_single_exec<Info>)
+        flags |= Process::ProcessFlags::SupportsState;
+
+      return flags;
     }
   }
 };
@@ -516,16 +522,25 @@ template <typename Info>
 struct MessageBusWrapperFromUi
 {
 };
+
+namespace
+{
+struct dummy_ui_callback
+{
+  void operator()(const QByteArray& arr) noexcept { }
+};
+}
+
 template <avnd::has_processor_to_gui_bus Info>
 struct MessageBusWrapperToUi<Info>
 {
-  std::function<void(QByteArray)> to_ui;
+  std::function<void(QByteArray)> to_ui = dummy_ui_callback{};
 };
 
 template <avnd::has_gui_to_processor_bus Info>
 struct MessageBusWrapperFromUi<Info>
 {
-  std::function<void(QByteArray)> from_ui;
+  std::function<void(QByteArray)> from_ui = dummy_ui_callback{};
 };
 
 template <typename Info>
@@ -549,15 +564,6 @@ public:
 
     avnd::port_visit_dispatcher<Info>(
         InletInitFunc<Info>{*this, m_inlets}, OutletInitFunc<Info>{*this, m_outlets});
-
-    if constexpr(requires { this->from_ui; })
-    {
-      this->from_ui = [](const QByteArray& arr) {};
-    }
-    if constexpr(requires { this->to_ui; })
-    {
-      this->to_ui = [](const QByteArray& arr) {};
-    }
   }
   ProcessModel(
       const TimeVal& duration, const QString& custom,
@@ -570,15 +576,6 @@ public:
     avnd::port_visit_dispatcher<Info>(
         InletInitFunc<Info>{*this, m_inlets}, OutletInitFunc<Info>{*this, m_outlets});
 
-    if constexpr(requires { this->from_ui; })
-    {
-      this->from_ui = [](const QByteArray& arr) {};
-    }
-    if constexpr(requires { this->to_ui; })
-    {
-      this->to_ui = [](const QByteArray& arr) {};
-    }
-
     if constexpr(avnd::file_input_introspection<Info>::size > 0)
     {
       constexpr auto idx = avnd::file_input_introspection<Info>::index_to_field_index(0);
